package javax.imageio;

import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.image.Raster;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.IOException;
import java.awt.Dimension;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.event.IIOWriteProgressListener;
import javax.imageio.event.IIOWriteWarningListener;
import java.util.List;
import java.util.Locale;
import javax.imageio.spi.ImageWriterSpi;

public abstract class ImageWriter implements ImageTranscoder
{
    protected ImageWriterSpi originatingProvider;
    protected Object output;
    protected Locale[] availableLocales;
    protected Locale locale;
    protected List<IIOWriteWarningListener> warningListeners;
    protected List<Locale> warningLocales;
    protected List<IIOWriteProgressListener> progressListeners;
    private boolean abortFlag;
    
    protected ImageWriter(ImageWriterSpi originatingProvider) {
        this.originatingProvider = null;
        this.output = null;
        this.availableLocales = null;
        this.locale = null;
        this.warningListeners = null;
        this.warningLocales = null;
        this.progressListeners = null;
        this.abortFlag = false;
        this.originatingProvider = originatingProvider;
    }
    
    public ImageWriterSpi getOriginatingProvider() {
        return this.originatingProvider;
    }
    
    public void setOutput(Object output) {
        if (output != null) {
            ImageWriterSpi originatingProvider = this.getOriginatingProvider();
            if (originatingProvider != null) {
                Class[] outputTypes = originatingProvider.getOutputTypes();
                boolean n = false;
                int i = 0;
                while (i < outputTypes.length) {
                    if (outputTypes[i].isInstance(output)) {
                        n = true;
                        break;
                    }
                    ++i;
                }
                if (n == 0) {
                    throw new IllegalArgumentException("Illegal output type!");
                }
            }
        }
        this.output = output;
    }
    
    public Object getOutput() {
        return this.output;
    }
    
    public Locale[] getAvailableLocales() {
        return (this.availableLocales == null) ? null : ((Locale[])(Locale[])this.availableLocales.clone());
    }
    
    public void setLocale(Locale locale) {
        if (locale != null) {
            Locale[] availableLocales = this.getAvailableLocales();
            boolean n = false;
            if (availableLocales != null) {
                int i = 0;
                while (i < availableLocales.length) {
                    if (locale.equals(availableLocales[i])) {
                        n = true;
                        break;
                    }
                    ++i;
                }
            }
            if (n == 0) {
                throw new IllegalArgumentException("Invalid locale!");
            }
        }
        this.locale = locale;
    }
    
    public Locale getLocale() {
        return this.locale;
    }
    
    public ImageWriteParam getDefaultWriteParam() {
        return new ImageWriteParam(this.getLocale());
    }
    
    public abstract IIOMetadata getDefaultStreamMetadata(ImageWriteParam p0);
    
    public abstract IIOMetadata getDefaultImageMetadata(ImageTypeSpecifier p0, ImageWriteParam p1);
    
    @Override
    public abstract IIOMetadata convertStreamMetadata(IIOMetadata p0, ImageWriteParam p1);
    
    @Override
    public abstract IIOMetadata convertImageMetadata(IIOMetadata p0, ImageTypeSpecifier p1, ImageWriteParam p2);
    
    public int getNumThumbnailsSupported(ImageTypeSpecifier imageTypeSpecifier, ImageWriteParam imageWriteParam, IIOMetadata iioMetadata, IIOMetadata iioMetadata2) {
        return 0;
    }
    
    public Dimension[] getPreferredThumbnailSizes(ImageTypeSpecifier imageTypeSpecifier, ImageWriteParam imageWriteParam, IIOMetadata iioMetadata, IIOMetadata iioMetadata2) {
        return null;
    }
    
    public boolean canWriteRasters() {
        return false;
    }
    
    public abstract void write(IIOMetadata p0, IIOImage p1, ImageWriteParam p2) throws IOException;
    
    public void write(IIOImage iioImage) throws IOException {
        this.write(null, iioImage, null);
    }
    
    public void write(RenderedImage renderedImage) throws IOException {
        this.write(null, new IIOImage(renderedImage, null, null), null);
    }
    
    private void unsupported() {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        throw new UnsupportedOperationException("Unsupported write variant!");
    }
    
    public boolean canWriteSequence() {
        return false;
    }
    
    public void prepareWriteSequence(IIOMetadata iioMetadata) throws IOException {
        this.unsupported();
    }
    
    public void writeToSequence(IIOImage iioImage, ImageWriteParam imageWriteParam) throws IOException {
        this.unsupported();
    }
    
    public void endWriteSequence() throws IOException {
        this.unsupported();
    }
    
    public boolean canReplaceStreamMetadata() throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void replaceStreamMetadata(IIOMetadata iioMetadata) throws IOException {
        this.unsupported();
    }
    
    public boolean canReplaceImageMetadata(int n) throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void replaceImageMetadata(int n, IIOMetadata iioMetadata) throws IOException {
        this.unsupported();
    }
    
    public boolean canInsertImage(int n) throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void writeInsert(int n, IIOImage iioImage, ImageWriteParam imageWriteParam) throws IOException {
        this.unsupported();
    }
    
    public boolean canRemoveImage(int n) throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void removeImage(int n) throws IOException {
        this.unsupported();
    }
    
    public boolean canWriteEmpty() throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void prepareWriteEmpty(IIOMetadata iioMetadata, ImageTypeSpecifier imageTypeSpecifier, int n, int n2, IIOMetadata iioMetadata2, List<? extends BufferedImage> list, ImageWriteParam imageWriteParam) throws IOException {
        this.unsupported();
    }
    
    public void endWriteEmpty() throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        throw new IllegalStateException("No call to prepareWriteEmpty!");
    }
    
    public boolean canInsertEmpty(int n) throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void prepareInsertEmpty(int n, ImageTypeSpecifier imageTypeSpecifier, int n2, int n3, IIOMetadata iioMetadata, List<? extends BufferedImage> list, ImageWriteParam imageWriteParam) throws IOException {
        this.unsupported();
    }
    
    public void endInsertEmpty() throws IOException {
        this.unsupported();
    }
    
    public boolean canReplacePixels(int n) throws IOException {
        if (this.getOutput() == null) {
            throw new IllegalStateException("getOutput() == null!");
        }
        return false;
    }
    
    public void prepareReplacePixels(int n, Rectangle rectangle) throws IOException {
        this.unsupported();
    }
    
    public void replacePixels(RenderedImage renderedImage, ImageWriteParam imageWriteParam) throws IOException {
        this.unsupported();
    }
    
    public void replacePixels(Raster raster, ImageWriteParam imageWriteParam) throws IOException {
        this.unsupported();
    }
    
    public void endReplacePixels() throws IOException {
        this.unsupported();
    }
    
    public synchronized void abort() {
        this.abortFlag = true;
    }
    
    protected synchronized boolean abortRequested() {
        return this.abortFlag;
    }
    
    protected synchronized void clearAbortRequest() {
        this.abortFlag = false;
    }
    
    public void addIIOWriteWarningListener(IIOWriteWarningListener iioWriteWarningListener) {
        if (iioWriteWarningListener == null) {
            return;
        }
        this.warningListeners = ImageReader.addToList(this.warningListeners, iioWriteWarningListener);
        this.warningLocales = ImageReader.addToList(this.warningLocales, this.getLocale());
        return;
    }
    
    public void removeIIOWriteWarningListener(IIOWriteWarningListener iioWriteWarningListener) {
        if (iioWriteWarningListener == null || this.warningListeners == null) {
            return;
        }
        int index = this.warningListeners.indexOf(iioWriteWarningListener);
        if (index != -1) {
            this.warningListeners.remove(index);
            this.warningLocales.remove(index);
            if (this.warningListeners.size() == 0) {
                this.warningListeners = null;
                this.warningLocales = null;
            }
        }
        return;
    }
    
    public void removeAllIIOWriteWarningListeners() {
        this.warningListeners = null;
        this.warningLocales = null;
    }
    
    public void addIIOWriteProgressListener(IIOWriteProgressListener iioWriteProgressListener) {
        if (iioWriteProgressListener == null) {
            return;
        }
        this.progressListeners = ImageReader.addToList(this.progressListeners, iioWriteProgressListener);
        return;
    }
    
    public void removeIIOWriteProgressListener(IIOWriteProgressListener iioWriteProgressListener) {
        if (iioWriteProgressListener == null || this.progressListeners == null) {
            return;
        }
        this.progressListeners = ImageReader.removeFromList(this.progressListeners, iioWriteProgressListener);
        return;
    }
    
    public void removeAllIIOWriteProgressListeners() {
        this.progressListeners = null;
    }
    
    protected void processImageStarted(int n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).imageStarted(this, n);
        }
        return;
    }
    
    protected void processImageProgress(float n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).imageProgress(this, n);
        }
        return;
    }
    
    protected void processImageComplete() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).imageComplete(this);
        }
        return;
    }
    
    protected void processThumbnailStarted(int n, int n2) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).thumbnailStarted(this, n, n2);
        }
        return;
    }
    
    protected void processThumbnailProgress(float n) {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).thumbnailProgress(this, n);
        }
        return;
    }
    
    protected void processThumbnailComplete() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).thumbnailComplete(this);
        }
        return;
    }
    
    protected void processWriteAborted() {
        if (this.progressListeners == null) {
            return;
        }
        for (int size = this.progressListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteProgressListener)this.progressListeners.get(i)).writeAborted(this);
        }
        return;
    }
    
    protected void processWarningOccurred(int n, String s) {
        if (this.warningListeners == null) {
            return;
        }
        if (s == null) {
            throw new IllegalArgumentException("warning == null!");
        }
        for (int size = this.warningListeners.size(), i = 0; i < size; ++i) {
            ((IIOWriteWarningListener)this.warningListeners.get(i)).warningOccurred(this, n, s);
        }
        return;
    }
    
    protected void processWarningOccurred(int n, String s, String key) {
        if (this.warningListeners == null) {
            return;
        }
        if (s == null) {
            throw new IllegalArgumentException("baseName == null!");
        }
        if (key == null) {
            throw new IllegalArgumentException("keyword == null!");
        }
        for (int size = this.warningListeners.size(), i = 0; i < size; ++i) {
            IIOWriteWarningListener iioWriteWarningListener = (IIOWriteWarningListener)this.warningListeners.get(i);
            Locale default1 = (Locale)this.warningLocales.get(i);
            if (default1 == null) {
                default1 = Locale.getDefault();
            }
            ClassLoader loader = (ClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    return Thread.currentThread().getContextClassLoader();
                }
            });
            ResourceBundle resourceBundle;
            try {
                resourceBundle = ResourceBundle.getBundle(s, default1, loader);
            }
            catch (MissingResourceException ex) {
                try {
                    resourceBundle = ResourceBundle.getBundle(s, default1);
                }
                catch (MissingResourceException ex2) {
                    throw new IllegalArgumentException("Bundle not found!");
                }
            }
            String string;
            try {
                string = resourceBundle.getString(key);
            }
            catch (ClassCastException ex3) {
                throw new IllegalArgumentException("Resource is not a String!");
            }
            catch (MissingResourceException ex4) {
                throw new IllegalArgumentException("Resource is missing!");
            }
            iioWriteWarningListener.warningOccurred(this, n, string);
        }
        return;
    }
    
    public void reset() {
        this.setOutput(null);
        this.setLocale(null);
        this.removeAllIIOWriteWarningListeners();
        this.removeAllIIOWriteProgressListeners();
        this.clearAbortRequest();
    }
    
    public void dispose() {
    }
}
